"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const log4js = require("log4js");
const cluster = require("cluster");
const logger_background_config_1 = require("../config/logger.background.config");
const background_config_1 = require("../config/background.config");
const dispatcher = require("./dispatcher.background");
log4js.configure(logger_background_config_1.LOGGER_CONFIG2);
let logger = log4js.getLogger("Background");
function ExecuteTask() {
    if (cluster.isMaster) {
        logger.info('Proceso principal', process.pid);
        console.log('Proceso principal', process.pid);
        Main();
    }
    else {
        logger.info('Proceso secundario', process.pid);
        console.log('Proceso secundario', process.pid);
        Worker();
    }
}
function Main() {
    try {
        let mainProcess = process.pid;
        let taskBackgroundProcessesByPid = [];
        let supervisorWorker = cluster.fork();
        let supervisorPidBackgroundProcesses = GenerateSupervisor(supervisorWorker, taskBackgroundProcessesByPid);
        cluster.on('message', (worker, message) => {
            if (message?.topic === 'log4js:message'
                &&
                    message.data)
                return;
            logger.info('Nueva solicitud de ejecucion', JSON.stringify(message));
            console.log('Nueva solicitud de ejecucion', JSON.stringify(message));
            if (message?.id === background_config_1.RELOAD_TASK
                &&
                    message.requiresStopping) {
                process.kill(mainProcess);
                return;
            }
            let totalTaskbackgroundProcessesByPid = (taskBackgroundProcessesByPid?.length ? taskBackgroundProcessesByPid.filter(item => !item.isOver) : []).length;
            let currentAvailableProcess = background_config_1.AVAILABLE_PROCESS - totalTaskbackgroundProcessesByPid > 0 ? background_config_1.AVAILABLE_PROCESS - totalTaskbackgroundProcessesByPid : 0;
            logger.info('Numero de procesos disponibles', (background_config_1.AVAILABLE_PROCESS - totalTaskbackgroundProcessesByPid));
            if (message?.id && message?.requiresReload) {
                let currentBackgroundProcessesByTask = message.processor
                    &&
                        message.id !== background_config_1.RELOAD_TASK ?
                    taskBackgroundProcessesByPid.filter(item => item.backgrounds?.length && item.backgrounds.some(item => item.id === message.id))
                    :
                        message.id === background_config_1.RELOAD_TASK ?
                            taskBackgroundProcessesByPid
                            :
                                [];
                let orderedTaskbackgroundProcesses = {};
                logger.info('Solicitud para recargar procesos pegados', JSON.stringify(message), 'procesos a eliminar', (currentBackgroundProcessesByTask?.length) || 0);
                if (currentBackgroundProcessesByTask?.length) {
                    currentBackgroundProcessesByTask.forEach(item => {
                        orderedTaskbackgroundProcesses[item.pid] = item?.backgrounds?.length || 0;
                        item.backgrounds = [];
                        KillProcess(cluster, item.pid, item, taskBackgroundProcessesByPid);
                    });
                }
                logger.info('Tarea pegadas', JSON.stringify(message), 'Procesos', currentBackgroundProcessesByTask?.length || 0, 'Ejecuciones por proceso', JSON.stringify(orderedTaskbackgroundProcesses));
            }
            else if (message?.id && message?.queue && !message.isOver) {
                let currentBackgroundProcessesByTask = !message.processor ?
                    taskBackgroundProcessesByPid.filter(item => item.isMain && !item.isOver)
                    :
                        taskBackgroundProcessesByPid.filter(item => item.backgrounds?.length && item.backgrounds.some(item => item.id === message.id) && !item.isOver);
                let totalBackgroundProcessByTask = currentBackgroundProcessesByTask?.length || 0;
                let totalBackgrounds = currentBackgroundProcessesByTask?.length ? currentBackgroundProcessesByTask.reduce((value, item) => { return value + item.backgrounds.filter(value => value.id === message.id && value.queue === message.queue).length; }, 0) : 0;
                totalBackgrounds = message.rate - totalBackgrounds;
                logger.info(message.id, 'Numero de procesos en ejecucion', totalBackgroundProcessByTask, 'Total de hilos', message.rate, 'Hilos faltantes', totalBackgrounds);
                if (totalBackgrounds > 0) {
                    logger.info('Solicitud para generacion de nuevos procesos/hilos', JSON.stringify(message));
                    let newBackgroundProcessByTask = !currentAvailableProcess ? 0 :
                        currentAvailableProcess - ((message.processor || background_config_1.DEFAULT_BACKGROUND_SUPERVISOR_CPU) - totalBackgroundProcessByTask) >= 0 ? (message.processor || background_config_1.DEFAULT_BACKGROUND_SUPERVISOR_CPU) - totalBackgroundProcessByTask :
                            0;
                    logger.info('Nuevos procesos a crear', newBackgroundProcessByTask);
                    logger.info('Nuevos hilos a crear', totalBackgrounds);
                    let position = 0;
                    let orderedTaskbackgroundProcesses = {};
                    for (let i = 0; i < totalBackgrounds; i++) {
                        let currentWorker;
                        let newBackground = {
                            id: message.id,
                            pid: null,
                            processor: message.processor,
                            queue: message.queue,
                            rate: message.rate,
                            isOver: message.isOver
                        };
                        if (newBackgroundProcessByTask) {
                            currentWorker = cluster.fork();
                            let newBackgroundProcess = {
                                pid: currentWorker.process.pid,
                                isMain: !message.processor,
                                backgrounds: [newBackground]
                            };
                            newBackground.pid = currentWorker.process.pid;
                            newBackground.isFirst = true;
                            taskBackgroundProcessesByPid.push(newBackgroundProcess);
                            currentBackgroundProcessesByTask.push(newBackgroundProcess);
                            logger.info('Nueva ejecucion en proceso nuevo', newBackground.pid);
                            currentWorker.send(newBackground);
                            --newBackgroundProcessByTask;
                        }
                        else {
                            let pid = currentBackgroundProcessesByTask[position] ?
                                currentBackgroundProcessesByTask[position].pid
                                :
                                    currentBackgroundProcessesByTask[0] ?
                                        currentBackgroundProcessesByTask[0].pid
                                        :
                                            null;
                            position = currentBackgroundProcessesByTask[position] ? position : 0;
                            currentWorker = Object.values(cluster.workers).find(s => s.process.pid === pid);
                            if (currentWorker) {
                                newBackground.pid = currentWorker.process.pid;
                                currentBackgroundProcessesByTask[position].backgrounds.push(newBackground);
                                logger.info('Nueva ejecucion en procesos existente', newBackground.pid);
                                currentWorker.send(newBackground);
                            }
                            else {
                                logger.info('No existe worker para', pid);
                            }
                            orderedTaskbackgroundProcesses[pid] = currentBackgroundProcessesByTask && currentBackgroundProcessesByTask[position] && currentBackgroundProcessesByTask[position].backgrounds && currentBackgroundProcessesByTask[position].backgrounds.length || 0;
                            position = position < currentBackgroundProcessesByTask.length ? position + 1 : 0;
                        }
                    }
                    logger.info('Tarea', JSON.stringify(message), 'Procesos', currentBackgroundProcessesByTask?.length || 0, 'Ejecuciones por proceso', JSON.stringify(orderedTaskbackgroundProcesses));
                }
            }
            else if (message?.id && message?.isOver) {
                logger.info('Solicitud de baja para proceso/ejecucion', JSON.stringify(message));
                let currentBackgroundsProcess = taskBackgroundProcessesByPid.find(item => item.pid === message.pid);
                if (currentBackgroundsProcess?.backgrounds.length) {
                    currentBackgroundsProcess.backgrounds.splice(currentBackgroundsProcess.backgrounds.findIndex(item => item.id === message.id && item.queue === message.queue), 1);
                    if (!currentBackgroundsProcess.backgrounds || !currentBackgroundsProcess.backgrounds.length) {
                        KillProcess(cluster, message.pid, currentBackgroundsProcess, taskBackgroundProcessesByPid);
                    }
                }
                else if (currentBackgroundsProcess) {
                    KillProcess(cluster, message.pid, currentBackgroundsProcess, taskBackgroundProcessesByPid);
                }
            }
            else {
                logger.error(`El worker ${worker.process.pid} no mapeado envió mensaje`, JSON.stringify(message || {}), 'debe ser de tipo TaskBackground');
            }
            if (!taskBackgroundProcessesByPid.find(s => s.pid === worker.process.pid)) {
                if (message?.topic === 'log4js:message')
                    return;
                logger.error(`El worker ${worker.process.pid} Eliminado por no pertenecer a los procesos guardados`, JSON.stringify(message || {}));
                worker.kill();
                process.kill(worker.process.pid);
            }
        });
        cluster.on('exit', (worker, code, signal) => {
            let currentBackgroundProcess = taskBackgroundProcessesByPid.find(x => x.pid === worker.process.pid);
            if (currentBackgroundProcess) {
                if (currentBackgroundProcess.isOver) {
                    logger.info('Exit Solicitud de baja para proceso/ejecucion', worker.process.pid);
                    let killMainBackgroundProcess = false;
                    killMainBackgroundProcess = worker.process.pid === supervisorPidBackgroundProcesses || killMainBackgroundProcess;
                    taskBackgroundProcessesByPid.splice(taskBackgroundProcessesByPid.findIndex(x => x.pid === worker.process.pid), 1);
                    if (killMainBackgroundProcess) {
                        let tempWorker = cluster.fork();
                        supervisorPidBackgroundProcesses = GenerateSupervisor(tempWorker, taskBackgroundProcessesByPid);
                    }
                }
                else {
                    logger.info('Exit solicitud de baja para proceso/ejecucion sin autorizacion', worker.process.pid);
                    let killMainBackgroundProcess = false;
                    killMainBackgroundProcess = worker.process.pid === supervisorPidBackgroundProcesses || killMainBackgroundProcess;
                    let killBackgrounds = [];
                    if (currentBackgroundProcess.backgrounds?.length) {
                        currentBackgroundProcess.backgrounds.forEach(itemBackground => {
                            if (!killBackgrounds.find(value => value.id === itemBackground.id && value.queue === itemBackground.queue)) {
                                killBackgrounds.push(JSON.parse(JSON.stringify(itemBackground)));
                            }
                        });
                    }
                    taskBackgroundProcessesByPid.splice(taskBackgroundProcessesByPid.findIndex(x => x.pid === worker.process.pid), 1);
                    let tempWorker = killMainBackgroundProcess ? cluster.fork() : Object.values(cluster.workers).find(s => s.process.pid === supervisorPidBackgroundProcesses);
                    if (killMainBackgroundProcess) {
                        supervisorPidBackgroundProcesses = GenerateSupervisor(tempWorker, taskBackgroundProcessesByPid);
                    }
                    if (killBackgrounds?.length) {
                        killBackgrounds.forEach(item => {
                            if (item.id === background_config_1.BACKGROUND_SUPERVISOR.find(value => value === item.id))
                                return;
                            item.pid = null;
                            tempWorker.process.send(item);
                        });
                    }
                }
            }
            else {
                logger.info(`El worker ${worker.process.pid} fue eliminado`);
            }
        });
    }
    catch (error) {
        logger.info('Finalizo con errores el proceso principal (Main)', process.pid);
    }
}
function Worker() {
    process.on('message', async function (msg) {
        let isFirst = msg?.isFirst;
        let taskId = msg?.id;
        let queue = msg?.queue;
        let threadDate = new Date();
        if (taskId && queue) {
            if (isFirst) {
                await dispatcher.StartService();
                logger.info('-------------Tengo conexion--------', taskId, queue, process.pid, threadDate);
            }
            logger.info('Inicia subproceso', taskId, queue, process.pid, threadDate);
            console.log('Inicia subproceso', taskId, queue, process.pid, threadDate);
            await dispatcher.ExecuteTask(taskId, queue, process.pid);
            logger.info('Termina el subproceso', taskId, queue, process.pid, threadDate);
            console.log('Termina el subproceso', taskId, queue, process.pid, threadDate);
            msg.isOver = true;
            process.send(msg);
        }
    });
}
function GenerateSupervisor(supervisorWorker, taskBackgroundProcessesByPid) {
    let supervisorBackgroundProcess = {
        pid: supervisorWorker.process.pid,
        isMain: true,
        backgrounds: []
    };
    if (background_config_1.BACKGROUND_SUPERVISOR?.length) {
        background_config_1.BACKGROUND_SUPERVISOR.forEach((item, index) => {
            if (background_config_1.DISABLED_TASK.includes(item))
                return;
            let supervisorBackground = {
                id: item,
                pid: supervisorWorker.process.pid,
                processor: null,
                queue: item,
                rate: background_config_1.SUPERVISOR_RATE,
                isOver: false,
                isFirst: index === 0
            };
            supervisorBackgroundProcess.backgrounds.push(supervisorBackground);
            supervisorWorker.send(supervisorBackground);
        });
        taskBackgroundProcessesByPid.push(supervisorBackgroundProcess);
    }
    return supervisorWorker.process.pid;
}
function KillProcess(currentCluster, pid, backgroundsProcess, taskBackgroundProcessesByPid) {
    let currentWorker = Object.values(currentCluster.workers).find((s) => s.process.pid === pid);
    backgroundsProcess.isOver = true;
    if (currentWorker) {
        currentWorker.kill();
        process.kill(currentWorker.process.pid);
    }
    else {
        taskBackgroundProcessesByPid.splice(taskBackgroundProcessesByPid.findIndex(x => x.pid === pid), 1);
    }
    return true;
}
ExecuteTask();
