"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteAdvancedAIIntent = exports.SaveAdvancedAIIntent = exports.GetAdvancedAIIntents = exports.AdvancedAIIntentMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function AdvancedAIIntentMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AdvancedAIIntentMSSql = AdvancedAIIntentMSSql;
function GetAdvancedAIIntents(vcc, searchCriteria, searchExpression, intentIds) {
    let logLabel = `[AdvancedAIIntentMSSql::GetAdvancedAIIntents] [vcc => ${vcc}]: `;
    if (searchCriteria && searchExpression)
        logger.info(`${logLabel}[search_criteria::search_expression => ${searchCriteria}::${searchExpression}]`);
    if (intentIds?.length)
        logger.info(`${logLabel}[appIds => ${intentIds}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .input('IntentIds', sql.NVarChar(sql.MAX), intentIds)
        .execute('GetAdvancedAIIntents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info(`${logLabel}getting recordset: `, recordset?.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAdvancedAIIntents = GetAdvancedAIIntents;
function SaveAdvancedAIIntent(vcc, intent, currentUser) {
    let logLabel = `[AdvancedAIIntentMSSql::SaveAdvancedAIIntent] [vcc::name => ${vcc}::${intent.name}]: `;
    if (intent.id)
        logger.info(`${logLabel}[appId => ${intent.id}]`);
    if (intent.prompt)
        logger.info(`${logLabel}[prompt => ${intent.prompt}]`);
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), intent.appId)
        .input('Id', sql.VarChar(50), intent.id)
        .input('Name', sql.VarChar(50), intent.name)
        .input('Prompt', sql.VarChar(1000), intent.prompt)
        .input('CurrentUser', sql.VarChar(50), currentUser)
        .execute('SaveAdvancedAIIntent')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveAdvancedAIIntent = SaveAdvancedAIIntent;
function DeleteAdvancedAIIntent(vcc, intent, currentUser) {
    let logLabel = `[AdvancedAIIntentMSSql::DeleteAdvancedAIIntent] [vcc::name => ${vcc}::${intent.name}]: `;
    if (intent.id)
        logger.info(`${logLabel}[id => ${intent.id}]`);
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), intent.appId)
        .input('Id', sql.VarChar(50), intent.id)
        .execute('DeleteAdvancedAIIntent')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteAdvancedAIIntent = DeleteAdvancedAIIntent;
