"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteAdvancedAI = exports.SaveAdvancedAI = exports.GetAdvancedAI = exports.GetAdvancedAIs = exports.AdvancedAIMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function AdvancedAIMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AdvancedAIMSSql = AdvancedAIMSSql;
function GetAdvancedAIs(vcc, searchCriteria, searchExpression) {
    let logLabel = `[${AdvancedAIMSSql.name}::${GetAdvancedAIs.name}] [vcc => ${vcc}]: `;
    if (searchCriteria && searchExpression)
        logger.info(`${logLabel}[search_criteria::search_expression => ${searchCriteria}::${searchExpression}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .execute('GetAdvancedAIs')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info(`${logLabel}getting recordset: `, recordset?.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAdvancedAIs = GetAdvancedAIs;
function GetAdvancedAI(vcc, searchCriteria, searchExpression) {
    let logLabel = `[${AdvancedAIMSSql.name}::${GetAdvancedAI.name}] [vcc::${searchCriteria} => ${vcc}::${searchExpression}]: `;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .execute('GetAdvancedAI')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        let advancedAI = (recordset?.length > 0 ? recordset[0] : null);
        if (advancedAI && sqlResult?.recordsets?.length > 1) {
            let intents = (sqlResult.recordsets?.length > 1 ? sqlResult.recordsets[1] : []);
            let examples = (sqlResult.recordsets?.length > 2 ? sqlResult.recordsets[2] : []);
            intents.forEach(i => {
                i.examples = examples.filter(e => e.intentId === i.id);
            });
            advancedAI.intents = intents;
        }
        logger.info(`${logLabel}getting recordset:`, advancedAI);
        return Promise.resolve(advancedAI);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAdvancedAI = GetAdvancedAI;
function SaveAdvancedAI(vcc, advancedAI, currentUser) {
    let logLabel = `[${AdvancedAIMSSql.name}::${SaveAdvancedAI.name}] [vcc::id => ${vcc}::${advancedAI.id}]: `;
    if (advancedAI.description)
        logger.info(`${logLabel}[description => ${advancedAI.description}]`);
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), advancedAI.id)
        .input('Description', sql.VarChar(500), advancedAI.description)
        .input('CurrentUser', sql.VarChar(50), currentUser)
        .execute('SaveAdvancedAI')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveAdvancedAI = SaveAdvancedAI;
function DeleteAdvancedAI(vcc, id, currentUser) {
    let logLabel = `[${AdvancedAIMSSql.name}::${DeleteAdvancedAI.name}] [vcc::id::user => ${vcc}::${id}::${currentUser}]: `;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), id)
        .execute('DeleteAdvancedAI')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteAdvancedAI = DeleteAdvancedAI;
