"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllAdvancedAIResultStorages = exports.DeleteAdvancedAIResult = exports.GetAdvancedAIResult = exports.SaveAdvancedAIResult = exports.GetAdvancedAIToken = exports.SaveAdvancedAIToken = exports.AdvancedAIRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function AdvancedAIRedis(app) {
    redisClient = app.get('redisClient');
}
exports.AdvancedAIRedis = AdvancedAIRedis;
function SaveAdvancedAIToken(vcc, token) {
    let hash = 'AdvancedAIToken';
    return redisClient.hsetAsync(hash, vcc, token)
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[AdvancedAIRedis::SaveAdvancedAIToken] getting error:', err, 'with vcc:', vcc);
        return Promise.resolve(false);
    });
}
exports.SaveAdvancedAIToken = SaveAdvancedAIToken;
function GetAdvancedAIToken(vcc) {
    let hash = 'AdvancedAIToken';
    return redisClient.hgetAsync(hash, vcc)
        .then(result => {
        return Promise.resolve(result);
    })
        .catch((err) => {
        logger.error('[AdvancedAIRedis::GetAdvancedAIToken] getting err value:', err, 'hash:', hash, 'vcc:', vcc);
        return Promise.resolve(null);
    });
}
exports.GetAdvancedAIToken = GetAdvancedAIToken;
function SaveAdvancedAIResult(interaction, index, results) {
    let logLabel = `[AdvancedAIRedis::${this.SaveAdvancedAIResult.name}] `;
    let regex = /\.(mp3|wav|ogg|json)$/;
    let interactionId = interaction.fileName.replace(regex, '');
    let hash = `AdvancedAIResult:${interaction.vcc}:${interactionId}`;
    return redisClient.hsetAsync(hash, `${interactionId}:${index}:${interaction.modelId}`, JSON.stringify({ index, results }))
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error(`${logLabel}getting error:`, err);
        return Promise.resolve(false);
    });
}
exports.SaveAdvancedAIResult = SaveAdvancedAIResult;
function GetAdvancedAIResult(key) {
    let logLabel = `[AdvancedAIRedis::${this.GetAdvancedAIResult.name}] `;
    let storage = 'AdvancedAIResult:';
    storage += key;
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let evaluationResults = [];
        try {
            let evaluationResult = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && evaluationResult) {
                for (let key in evaluationResult) {
                    let evaluation = JSON.parse(evaluationResult[key]);
                    evaluation.model = key.split(':')[2];
                    evaluationResults.push(evaluation);
                }
            }
        }
        catch (err) {
            logger.error(`${logLabel}getting err value:`, result);
            return Promise.resolve(null);
        }
        return evaluationResults;
    })
        .catch((err) => {
        logger.error(`${logLabel}getting err value:`, err);
        return Promise.resolve(err);
    });
}
exports.GetAdvancedAIResult = GetAdvancedAIResult;
function DeleteAdvancedAIResult(key) {
    let logLabel = `[AdvancedAIRedis::${this.DeleteAdvancedAIResult.name}] `;
    let storage = 'AdvancedAIResult:';
    storage += key;
    if (!key)
        return Promise.resolve(false);
    return redisClient.delAsync(storage)
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error(`${logLabel}getting error:`, err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteAdvancedAIResult = DeleteAdvancedAIResult;
function GetAllAdvancedAIResultStorages() {
    let logLabel = `[AdvancedAIRedis::${this.GetAllAdvancedAIResultStorages.name}] `;
    let config = redis_config_1.REDIS_OPTIONS;
    let pattern = `${config.prefix}AdvancedAIResult:*`;
    return redisClient.keysAsync([pattern])
        .then((result) => {
        if (result?.length > 1)
            logger.info(`${logLabel}Getting ${result.length} storages`);
        let executionCodes = [];
        result.forEach(r => {
            executionCodes.push(r.replace(`${config.prefix}AdvancedAIResult:`, ''));
        });
        executionCodes = executionCodes.sort();
        return Promise.resolve(executionCodes);
    })
        .catch((err) => {
        logger.error(`${logLabel}getting err value:`, err);
        return Promise.resolve(err);
    });
}
exports.GetAllAdvancedAIResultStorages = GetAllAdvancedAIResultStorages;
