"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetTranscriptionEngineAvailability = exports.SaveTranscriptionEngineAvailability = exports.GetTaskBackgroundStop = exports.DeleteTaskBackgroundStop = exports.SaveTaskBackgroundStop = exports.GetBillingCurrentBatches = exports.SaveBillingCurrentBatches = exports.TaskRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
const promises_shared_1 = require("../../shared/promises.shared");
let logger = log4js.getLogger('Redis');
let redisClient;
function TaskRedis(app) {
    redisClient = app.get('redisClient');
}
exports.TaskRedis = TaskRedis;
function SaveBillingCurrentBatches(data) {
    let redisKey = 'BillingCurrentBatches';
    return redisClient.hsetAsync(redisKey, 'Batches', JSON.stringify(data))
        .then(_ => {
        return redisClient.expireAsync(redisKey, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        return Promise.reject(err);
    });
}
exports.SaveBillingCurrentBatches = SaveBillingCurrentBatches;
function GetBillingCurrentBatches() {
    let logLabel = `[TaskRedis.task::GetBillingCurrentBatches] `;
    let redisKey = 'BillingCurrentBatches';
    return redisClient.hgetAsync(redisKey, 'Batches')
        .then((result) => {
        let billingCurrentBatechesData = null;
        try {
            billingCurrentBatechesData = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error(`${logLabel}Catching err value:`, err);
            return Promise.resolve(null);
        }
        return Promise.resolve(billingCurrentBatechesData);
    })
        .catch((err) => {
        logger.error(`${logLabel}Getting err value:`, err);
        return Promise.resolve(null);
    });
}
exports.GetBillingCurrentBatches = GetBillingCurrentBatches;
function SaveTaskBackgroundStop(queue, status) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!queue)
            throw 'Se intento consultar un registro en una clave nula';
        return redisClient.hSet('QueueStop', queue, status ? 1 : 0);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        console.dir(err);
        return Promise.resolve(false);
    });
}
exports.SaveTaskBackgroundStop = SaveTaskBackgroundStop;
function DeleteTaskBackgroundStop() {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return redisClient.del('QueueStop');
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        console.dir(err);
        return Promise.resolve(false);
    });
}
exports.DeleteTaskBackgroundStop = DeleteTaskBackgroundStop;
function GetTaskBackgroundStop(queue) {
    let status = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!queue)
            throw 'Se intento consultar un registro en una clave nula';
        return redisClient.hgetAsync('QueueStop', queue);
    })
        .then((value) => {
        if (!parseInt(value))
            return redisClient.hgetAsync('QueueStop', 'all');
        return Promise.resolve(value);
    })
        .then((value) => {
        return Promise.resolve(!!parseInt(value));
    })
        .catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.resolve(true);
    });
}
exports.GetTaskBackgroundStop = GetTaskBackgroundStop;
function SaveTranscriptionEngineAvailability(value) {
    let logLabel = `[TaskRedis.task::SaveTranscriptionEngineAvailability] `;
    let storage = 'TranscriptionEngineAvailability';
    let key = 'Deepgram';
    return redisClient.hsetAsync(storage, key, JSON.stringify(value))
        .then(_ => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE_ONE_HOUR);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error(`${logLabel}Getting err value:`, err);
        return Promise.resolve(false);
    });
}
exports.SaveTranscriptionEngineAvailability = SaveTranscriptionEngineAvailability;
function GetTranscriptionEngineAvailability() {
    let logLabel = `[TaskRedis.task::GetTranscriptionEngineAvailability] `;
    let storage = 'TranscriptionEngineAvailability';
    let key = 'Deepgram';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let availability = true;
        try {
            let value = (typeof result === 'string') ? JSON.parse(result) : result;
            availability = value;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(availability);
    })
        .catch((err) => {
        logger.error(`${logLabel}Getting err value:`, err);
        return Promise.resolve(null);
    });
}
exports.GetTranscriptionEngineAvailability = GetTranscriptionEngineAvailability;
