"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cluster_1 = require("cluster");
const os_1 = require("os");
const path = require('path');
const fs = require('fs');
const sql = require('mssql');
let cpuCount = os_1.default.cpus().length;
let cpuToProcess = 0;
let deletedPids = [];
console.log(`Número de CPUs disponibles ${cpuCount}`);
console.log(`PID principal: ${process.pid}`);
function addWorkersCluster(cpus) {
    cluster_1.default.setupMaster({
        exec: path.resolve(__dirname, 'serviceTesting.js')
    });
    for (let i = 0; i < cpus; i++) {
        cluster_1.default.fork();
    }
    cluster_1.default.on("exit", (worker, code, signal) => {
        console.log(`worker ${worker.process.pid} ha sido eliminado`);
        if (!deletedPids.includes(worker.process.pid)) {
            console.log("Leevantando otro worker");
            cluster_1.default.fork();
        }
    });
}
function killWorkerCluster(cpus) {
    let i = 0;
    for (let worker of Object.values(cluster_1.default.workers)) {
        i++;
        console.log(`Eliminando ${i} core: ${JSON.stringify(worker)}`);
        let processId = worker.process.pid;
        deletedPids.push(processId);
        worker.kill();
        process.kill(processId);
        if (i === cpus)
            break;
    }
}
function startProcess() {
    deletedPids = [];
    try {
        let data = fs.readFileSync('properties.txt', 'utf-8');
        let parametros = data.split('\n');
        let cpuCountFromParameter = Number(parametros[0]);
        if (cpuCountFromParameter === 0 || cpuCountFromParameter > cpuCount)
            throw "En número de CPUs no puede ser 0 o mayor al que se tiene";
        if (cpuCountFromParameter < cpuToProcess)
            killWorkerCluster(cpuToProcess - cpuCountFromParameter);
        if (cpuCountFromParameter > cpuToProcess)
            addWorkersCluster(cpuCountFromParameter - cpuToProcess);
        cpuToProcess = cpuCountFromParameter;
    }
    catch (err) {
        console.error('Error:', JSON.stringify(err));
        cpuToProcess = cpuCount;
    }
    console.log("Cantidad de CPUs permitidos para procesar:", cpuToProcess);
    setTimeout(startProcess, 10000);
}
startProcess();
let sqlConn;
let dbConfigMssql = {
    server: '10.171.0.21',
    database: 'InconcertOnlineStv',
    port: 1433,
    user: 'inconcert',
    password: 'InC0nc3rt!',
    trustServerCertificate: true,
};
const logFilePath = 'archivo.txt';
let interval = 3000;
function ConectDB() {
    return sql.connect(dbConfigMssql).then((result) => {
        sqlConn = result;
    });
}
function log(message) {
    const logMessage = `${new Date().toISOString()}: ${message}\n`;
    fs.appendFile(logFilePath, logMessage, (err) => {
        if (err)
            console.error('Error al escribir en el archivo de log:', err);
    });
}
function GetNewActivities(instance, currentStatus, status) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('CurrentStatus', sql.VarChar(50), currentStatus)
        .input('Status', sql.VarChar(50), status)
        .execute('GetNewActivities')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 0 ? sqlResult.recordsets[0] : null);
        let activities = (recordset && recordset.length > 0 ? recordset : []);
        return Promise.resolve(activities);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        console.error("Error:", errorMsg);
        return Promise.reject(err);
    }).finally(() => {
        if (sqlConn) {
            console.log("cerrando conxión");
            sqlConn.close();
        }
    });
    ;
}
function checkNewActivities() {
    let activities = [];
    let msg;
    return ConectDB()
        .then(() => {
        console.log('Conexión establecida con éxito.');
        return GetNewActivities('default', 'toProcess', 'inProcess');
    }).then(result => {
        if (result && result.length > 0) {
            activities = result;
            msg = `Se encontraron nuevos actividades registradas:', ${result.length}`;
            console.log(msg);
            log(msg);
        }
        else {
            msg = "No se encontraron registros/actividades nuevas";
            console.warn(msg);
            log(msg);
        }
        return Promise.resolve(activities);
    }).catch(err => {
        msg = `Error al consultar nuevas actividades: ${JSON.stringify(err)}`;
        console.error(msg);
        log(msg);
        return Promise.reject(err);
    });
}
function startCheckNewActivities() {
    checkNewActivities()
        .then(() => {
        setTimeout(startCheckNewActivities, interval);
    })
        .catch(err => {
        console.error('Error en el listener:', JSON.stringify(err));
        setTimeout(startCheckNewActivities, interval);
    });
}
startCheckNewActivities();
