"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdvancedAIService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const advanced_ai_model_1 = require("../models/advanced-ai.model");
const promises_shared_1 = require("../shared/promises.shared");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('ServerScripts');
function AdvancedAIService(app) {
    app.post('/api/advanced_ai/get_advanced_ais/', (req, res, next) => {
        let vcc = res.locals.vcc;
        let logLabel = `[${AdvancedAIService.name}::get_advanced_ais] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        let search_criteria = req.body.searchCriteria || '';
        let search_expression = req.body.searchExpression || '';
        let advancedAIs = [];
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAdvancedAIs(vcc, search_criteria, search_expression, null);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_ADVANCED_AIS_FAILED';
            logger.info(`${logLabel}getting advancedAIs from database:`, result.length);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_ADVANCED_AIS_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/advanced_ai/get/:id', (req, res, next) => {
        let vcc = res.locals.vcc;
        let advancedAI = new advanced_ai_model_1.AdvancedAI();
        advancedAI.id = req.params.id;
        let logLabel = `[${AdvancedAIService.name}::get] [vcc::modelId => ${vcc}::${advancedAI.id}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAdvancedAI(vcc, 'id', advancedAI.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_ADVANCED_AI_FAILED';
            advancedAI = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: advancedAI });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/advanced_ai/create', (req, res, next) => {
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let advancedAI = (req.body || {});
        let logLabel = `[${AdvancedAIService.name}::create] [vcc::advancedAI => ${vcc}::${advancedAI.id}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAdvancedAI(vcc, 'id', advancedAI.id);
        })
            .then(result => {
            if (result)
                throw 'SERVER_ERROR_ADVANCED_AI_NAME_ALREADY_EXISTS';
            return mssql_service_1.MSSql.SaveAdvancedAI(vcc, advancedAI, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SAVE_ADVANCED_AI_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_CREATE_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/advanced_ai/save', (req, res, next) => {
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let advancedAI = (req.body || {});
        let logLabel = `[${AdvancedAIService.name}::save] [vcc::id => ${vcc}::${advancedAI.id}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAdvancedAI(vcc, 'id', advancedAI.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_ADVANCED_AI_ID_NOT_FOUND';
            return SaveAdvancedAI(vcc, advancedAI, userId, result);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SAVE_ADVANCED_AI_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: true });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_CREATE_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/advanced_ai/delete/:id', (req, res, next) => {
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let advancedAIId = req.params.id;
        let currentBlockPromise = null;
        let logLabel = `[${AdvancedAIService.name}::delete] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_SEARCHING_ID_ON_DB';
            return mssql_service_1.MSSql.GetAdvancedAI(vcc, 'id', advancedAIId);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_DELETING_AAI_ON_DB';
            if (!result)
                throw 'SERVER_ERROR_ADVANCED_AI_NOT_FOUND';
            return mssql_service_1.MSSql.DeleteAdvancedAI(vcc, advancedAIId, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_DELETE_APP_ON_BD_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: true });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_SERVICE_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}Block: ${currentBlockPromise}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/advanced_ai/get_advanced_ai_intent_items', (req, res, next) => {
        let vcc = res.locals.vcc;
        let logLabel = `[AdvancedAIService::get_advanced_ai_intent_items] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAdvancedAIIntentItems(vcc);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_ADVANCED_AI_INTENT_ITEMS_FAILED';
            logger.info(`${logLabel}getting advanced_ai-itent-items from database:`, result.length);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_ADVANCED_AI_INTENT_ITEMS_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
}
exports.AdvancedAIService = AdvancedAIService;
function SaveAdvancedAI(vcc, advancedAI, currentUser, oldAdvancedAI) {
    let logLabel = `[AdvancedAIService::SaveAdvancedAI] [vcc::modelName::user => ${vcc}::${advancedAI.id}::${currentUser}]: `;
    logger.info(`${logLabel}appId: ${advancedAI.id}`);
    oldAdvancedAI ?? (oldAdvancedAI = new advanced_ai_model_1.AdvancedAIExtended());
    let intentsSummary = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.SaveAdvancedAI(vcc, advancedAI, currentUser);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_ADVANCED_AI_FAILED';
        let promiseArray = [];
        intentsSummary = arrays_shared_1.GetCompareItemsFromObjectArray(oldAdvancedAI.intents, advancedAI.intents, 'id');
        intentsSummary.old.forEach(i => {
            i.appId ?? (i.appId = advancedAI.id);
            promiseArray.push(mssql_service_1.MSSql.SaveAdvancedAIIntent(vcc, i, currentUser));
        });
        intentsSummary.new.forEach(i => {
            i.appId ?? (i.appId = advancedAI.id);
            promiseArray.push(mssql_service_1.MSSql.SaveAdvancedAIIntent(vcc, i, currentUser));
        });
        intentsSummary.removed.forEach(i => {
            i.appId ?? (i.appId = advancedAI.id);
            promiseArray.push(mssql_service_1.MSSql.DeleteAdvancedAIIntent(vcc, i, currentUser));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result.filter(r => !r).length)
            throw 'SERVER_ERROR_AT_LEAST_ONE_SAVE_INTENT_FAILED';
        let promiseArray = [];
        advancedAI.intents.forEach(i => {
            let intent = oldAdvancedAI.intents.find(oi => oi.id === i.id);
            if (intent) {
                let examplesSummary = arrays_shared_1.GetCompareItemsFromObjectArray(intent.examples, i.examples, 'id');
                examplesSummary.new.forEach(e => {
                    e.appId ?? (e.appId = advancedAI.id);
                    e.intentId ?? (e.intentId = i.id);
                    promiseArray.push(mssql_service_1.MSSql.SaveAdvancedAIExample(vcc, e, currentUser));
                });
                examplesSummary.removed.forEach(e => {
                    e.appId ?? (e.appId = advancedAI.id);
                    e.intentId ?? (e.intentId = i.id);
                    promiseArray.push(mssql_service_1.MSSql.DeleteAdvancedAIExample(vcc, e, currentUser));
                });
            }
            else
                i.examples.forEach(e => {
                    e.appId ?? (e.appId = advancedAI.id);
                    e.intentId ?? (e.intentId = i.id);
                    promiseArray.push(mssql_service_1.MSSql.SaveAdvancedAIExample(vcc, e, currentUser));
                });
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result.filter(r => !r).length)
            throw 'SERVER_ERROR_AT_LEAST_ONE_SAVE_EXAMPLE_FAILED';
        return Promise.resolve(true);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.resolve(false);
    });
}
